#!/bin/sh

module=drweb

check_os_release() {
    os=unknown

    if [ -f /etc/redhat-release ]; then
        if  grep -q '^Red Hat Enterprise Linux Server release 5' \
            /etc/redhat-release || \
            grep -q '^CentOS release 5' /etc/redhat-release; then
            os=rhel5
        elif grep -q '^Red Hat Enterprise Linux Server release 6' \
            /etc/redhat-release || \
            grep -q '^CentOS release 6' /etc/redhat-release; then
            os=rhel6
        fi
    fi

    echo $os
}

check_release() {
    uname -r | grep -q "$1"
}

check_arch() {
    uname -m | grep -q "^$1$"
}

unsupported() {
    echo 'Warning: Your system does not support fanotify;' \
        'you need to build and install kernel module manually' \
        'from the source codes.'
}

install_mod() {
    from="/opt/drweb.com/lib/modules/$1/extra"
    to="/lib/modules/`uname -r`/extra"

    if [ ! -d "$from" ]; then
        echo "Directory \"$from\" not found."
        exit 1
    fi


    while read x; do
        if ! grep -q "$x" /boot/System.map-`uname -r`; then
            unsupported
            return
        fi
    done < "$from/$module.ko.syms"

    mkdir -p "$to"
    ln -sf "$from/$module.ko" "$to"
    echo "$to/$module.ko" >> /var/opt/drweb.com/spider-kmod-symlinks

    echo "Installed \"$to/$module.ko\"."

    echo "Doing depmod ..."
    depmod

    if [ `uname -s` = Linux ] && \
        env LANG=C stat --printf %N /proc/1/exe | awk '{print $3}' | \
        grep /systemd >/dev/null; then
        service drweb-configd reload || true
    else
        /etc/init.d/drweb-configd reload || true
    fi

    echo 'Notice: Your system does not support fanotify;' \
        'the supplied kernel module will be used.'
}

check_fanotify() {
    have_fanotify=

    if [ -f /proc/config.gz ]; then
        zcat /proc/config.gz | grep -q CONFIG_FANOTIFY=y && have_fanotify=1
    elif [ -f "/boot/config-`uname -r`" ]; then
        grep -q CONFIG_FANOTIFY=y "/boot/config-`uname -r`" && have_fanotify=1
    fi

    if [ -n "$have_fanotify" ]; then
        glibc=$(ldd `which printf` 2>/dev/null | grep libc\\.so | \
            awk '{print $3}')
        if [ -f "$glibc" ]; then
            strings "$glibc" 2>/dev/null | grep -q fanotify_init || \
                have_fanotify=
        fi
    fi

    if [ -n "$have_fanotify" ]; then
        echo 'Notice: Your system supports fanotify;' \
            'the kernel module is not required.'
        return 0
    fi

    return 1
}

check_fanotify && exit 0

module_path="/lib/modules/`uname -r`//extra/$module.ko"
[ -f "$module_path" ] && mv "$module_path" "$module_path.drweb-old"

if [ -f /etc/redhat-release ]; then
    case `check_os_release` in
        rhel5)
            check_release '^2\.6\.18-.*el5$' && check_arch 'x86_64' && \
                install_mod '2.6.18-371.11.1.el5'

            check_release '^2\.6\.18-.*el5xen$' && check_arch 'x86_64' && \
                    install_mod '2.6.18-371.11.1.el5xen'

            check_release '^2\.6\.18-.*el5$' && check_arch 'i686' && \
                    install_mod '2.6.18-371.el5'

            check_release '^2\.6\.18-.*el5PAE$' && \
                install_mod '2.6.18-371.el5PAE'

            check_release '^2\.6\.18-.*el5xen$' && check_arch 'i686' && \
                install_mod '2.6.18-371.el5xen'
            ;;
        rhel6)
            check_release '^2\.6\.32-.*el6.i686$' && \
                install_mod '2.6.32-431.23.3.el6.i686'

            check_release '^2\.6\.32-.*el6.x86_64$' && \
                install_mod '2.6.32-431.23.3.el6.x86_64'
            ;;
        *)
            unsupported
            ;;
    esac
else
    unsupported
fi

exit 0
